@@ -1,12 +1,12 @@
 #!/usr/bin/perl -w
 #
-# $Id: parted2fdisk.pl 221 2005-12-15 08:48:11Z bcornec $
+# $Id: parted2fdisk.pl 1857 2008-01-11 14:28:40Z bruno $
 #
 # parted2fdisk: fdisk like interface for parted 
-# [developped for mindi/mondo http://mondorescue.berlios.de]
+# [developped for mindi/mondo http://www.mondorescue.org]
 #
 # Aims to be architecture independant (i386/ia64)
-# Tested on RHAS 2.1 ia64 - Mandrake 9.0 ia64 - RHAS 3.0 ia64
+# Tested on RHAS 2.1 ia64 - Mandrake 9.0 ia64 - RHAS 3.0 ia64 - SLES 10
 #
 # (c) Bruno Cornec <Bruno.Cornec@hp.com>
 # Licensed under the GPL
@@ -35,15 +35,15 @@ my $cylend;
 my %start;
 my %end;
 my %type;
-my %flags;
 my $arch;
 my $fake = 0;
+my $mega = 1048576;
 
 # Determine on which arch we're running
 if (defined ($ENV{ARCH})) {
 	$arch = $ENV{ARCH};
 } else {
-	$arch = `"uname -m"`;
+	$arch = `uname -m`;
 	chomp($arch);
 }
 
@@ -73,6 +73,7 @@ my %pid = (	"FAT" => "6",
 		"fat16" => "e",
 		"ext2" => "83",
 		"ext3" => "83",
+		"reiserfs" => "83",
 		"linux-swap" => "82",
 		"LVM" => "8e",
 		"" => "",
@@ -164,7 +165,7 @@ if ($arch =~ /^ia64/) {
 					if ((not (defined $cylstart)) || ($cylstart eq "")) {
 						if (defined $start{$part-1}) {
 							# in MB => cyl
-							$cylstart = sprintf("%d",$end{$part-1}*1048576/$un + 1);
+							$cylstart = sprintf("%d",$end{$part-1}*$mega/$un + 1);
 							print FLOG "no start cyl given for creation... assuming the following $cylstart\n";
 						} else {
 							print FLOG "no start cyl given for creation... assuming the following 1\n";
@@ -178,7 +179,7 @@ if ($arch =~ /^ia64/) {
 					if ($cylstart == 1) {
 						$start = 0.01;
 					} else {
-						$start = $cylstart* $un / 1048576 +0.001;
+						$start = $cylstart* $un / $mega + 0.001;
 					}
 					# this is a size in B/KB/MB/GB
 
@@ -193,16 +194,8 @@ if ($arch =~ /^ia64/) {
 					# to give cylinders
 					if ($cylend =~ /^\+/) {
 						$cylend =~ s/^\+//;
-						if ($cylend =~ /K$/) {
-							$cylend =~ s/K$//;
-							$cylend *= 1024;
-						} elsif ($cylend =~ /M$/) {
-							$cylend =~ s/M$//;
-							$cylend *= 1048576;
-						} elsif ($cylend =~ /G$/) {
-							$cylend =~ s/G$//;
-							$cylend *= 1073741824;
-						}
+						# Handles suffixes; return bytes
+						$cylend = decode_Bsuf($cylend,1);
 						# This gives the number of cyl
 						$cylend /= $un;
 						$cylend = sprintf("%d",$cylend);
@@ -212,7 +205,7 @@ if ($arch =~ /^ia64/) {
 					$cylend = $endmax if ($cylend > $endmax); 
 					print FLOG "end cyl : $cylend\n";
 					# parted needs MB
-					$end = $cylend * $un / 1048576;
+					$end = $cylend * $un / $mega;
 					print FLOG "n $l $part $cylstart $cylend => mkpart primary $start $end\n";
 					system "$parted -s $device mkpart primary ext2 $start $end\n" if ($fake == 0);
 				}
@@ -283,14 +276,14 @@ my $fargs = join(@ARGV);
 
 if ($args =~ /^-/) {
 	# -l or -s
-	open (FDISK, "$fdisk $fargs |") || die "Unable to read from $fdisk";
+	open (FDISK, "$fdisk $fargs 2>/dev/null |") || die "Unable to read from $fdisk";
 	while (<FDISK>) {
 		print;
 	}
 	close(FDISK);
 } else {
 	# Modification mode
-	open (FDISK, "| $fdisk $fargs") || die "Unable to modify through $fdisk";
+	open (FDISK, "| $fdisk $fargs 2>/dev/null") || die "Unable to modify through $fdisk";
 	while (<STDIN>) {
 		print FDISK;
 	}
@@ -341,6 +334,7 @@ my $n;
 my %cmt = (	"FAT" => "FAT",
 		"ext2" => "Linux",
 		"ext3" => "Linux",
+		"reiserfs" => "Linux",
 		"linux-swap" => "Linux swap",
 		"LVM" => "Linux LVM",
 		"fat16" => "fat16",
@@ -355,26 +349,28 @@ my $length;
 my $pid;
 my $cmt;
 format FLOG1 =
-@<<<<<<<<<<<< @>>>>>>>> @>>>>>>>> @>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+@<<<<<<<<<<<< @>>>>>>>>>> @>>>>>>>>>> @>>>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $part,        $mstart,   $mend,   $length,  $pid, $cmt
 .
 format FLOG2 =
 @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $part,
-              @>>>>>>>> @>>>>>>>> @>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+              @>>>>>>>>>> @>>>>>>>>>> @>>>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
               $mstart,   $mend,   $length,  $pid, $cmt
 .
 format STDOUT1 =
-@<<<<<<<<<<<< @>>>>>>>> @>>>>>>>> @>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+@<<<<<<<<<<<< @>>>>>>>>>> @>>>>>>>>>> @>>>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $part,        $mstart,   $mend,   $length,  $pid, $cmt
 .
 format STDOUT2 =
 @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $part,
-              @>>>>>>>> @>>>>>>>> @>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+              @>>>>>>>>>> @>>>>>>>>>> @>>>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
               $mstart,   $mend,   $length,  $pid, $cmt
 .
-#/dev/hda1             1     77579  39099374+  ee  EFI GPT
+#   Device Boot      Start         End      Blocks   Id  System
+#/dev/hda1               1       77579    39099374+  ee  EFI GPT
+
 
 #
 # Keep Fdisk headers
@@ -385,17 +381,19 @@ $un = get_un ($device,$wpart,$verbose);
 $endmax = get_max($device);
 
 # This will return MB
-get_parted ($device,$start,$end,\%type,\%flags);
+get_parted ($device,$start,$end,\%type);
 
 while (($n,$d) = each %type) {
 	# Print infos fdisk like
 	$part = ${device}.$n;
 	# start and end are in cylinder in fdisk format
 	# so return in MB * 1MB / what represents 1 cyl in B
-	$mstart = sprintf("%d",$$start{$n}*1048576/$un);
+	$mstart = sprintf("%d",$$start{$n}*$mega/$un);
 	$mstart = 1 if ($mstart < 1);
-	$mend = sprintf("%d",$$end{$n}*1048576/$un - 1);
+	$mstart = $endmax if ($mstart > $endmax);
+	$mend = sprintf("%d",$$end{$n}*$mega/$un - 1);
 	$mend = $endmax if ($mend > $endmax);
+	$mend = 1 if ($mend < 1);
 	# length is in 1K blocks
 	$length = sprintf("%d",($mend-$mstart+1)*$un/1024);
 	$pid = $pid{$type{$n}};
@@ -445,7 +443,7 @@ my $device = shift;
 my $max = 0;
 my $foo;
 
-open (FDISK, "$fdisk -l $device |") || die "Unable to read from $fdisk";
+open (FDISK, "$fdisk -l $device 2>/dev/null |") || die "Unable to read from $fdisk";
 while (<FDISK>) {
 	if ($_ =~ /heads/) {
 		chomp;
@@ -469,7 +467,7 @@ my $verbose = shift;
 my $un = 0;
 my $foo;
 
-open (FDISK, "$fdisk -l $device |") || die "Unable to read from $fdisk";
+open (FDISK, "$fdisk -l $device 2>/dev/null |") || die "Unable to read from $fdisk";
 while (<FDISK>) {
 	print if (($_ !~ /^\/dev\//) and (not (defined $wpart)) and ($verbose == 1));
 	if ($_ =~ /^Units/) {
@@ -485,6 +483,7 @@ return($un);
 
 # 
 # Parted gives info in MB
+# (depending on versions - 1.6.25.1 provides suffixes)
 #
 sub get_parted {
 
@@ -492,33 +491,93 @@ my $device = shift;
 my $start = shift;
 my $end = shift;
 my $type = shift;
-my $flags = shift;
+my $void;
 my $d;
 my $n;
+my $ret;
+my $mode;
+my $size;
+my $unit;
+
+open (PARTED, "$parted -v |") || die "Unable to read from $parted";
+$d = <PARTED>;
+print FLOG "$d";
+close(PARTED);
 
 open (PARTED, "$parted -s $device print |") || die "Unable to read from $parted";
 # Skip 3 first lines
 $d = <PARTED>;
 $d = <PARTED>;
 $d = <PARTED>;
+
+# depending on parted version, information given change:
+if ($d =~ /\bSize\b/) {
+	# SLES 10 parted >= 1.6.25
+	$mode=1;
+} else {
+	# RHEL 3 parted 1.6.3
+	# RHEL 4 parted 1.6.19
+	$mode=0;
+}
+print FLOG "mode: $mode\n";
 print FLOG "Got from parted: \n";
-print FLOG "Minor    Start       End     Filesystem                        Flags\n";
+print FLOG "Minor    Start       End     Filesystem\n";
 # Get info from each partition line
 while (($n,$d) = split(/\s/, <PARTED>,2)) {
 	chomp($d);
 	next if ($n !~ /^[1-9]/);
 	$d =~ s/^\s*//;
 	$d =~ s/\s+/ /g;
-	($$start{$n},$$end{$n},$$type{$n},$$flags{$n}) = split(/ /,$d);
+	if ($mode == 0) {
+		($$start{$n},$$end{$n},$$type{$n},$void) = split(/ /,$d);
+		$unit = 1;
+	} elsif ($mode == 1) {
+		($$start{$n},$$end{$n},$size,$$type{$n},$void) = split(/ /,$d);
+		$unit = $mega;
+	} else {
+		die "Undefined mode $mode";
+	}
 	$$start{$n} = "" if (not defined $$start{$n});
 	$$end{$n} = "" if (not defined $$end{$n});
 	$$type{$n} = "" if (not defined $$type{$n});
-	$$flags{$n} = "" if (not defined $$flags{$n});
-	print FLOG "$n      $$start{$n}      $$end{$n}     $$type{$n}  $$flags{$n}\n";
+	# Handles potential suffixes in latest parted version. Return MB
+	$ret = decode_Bsuf($$start{$n},$unit);
+	$$start{$n} = $ret;
+	$ret = decode_Bsuf($$end{$n},$unit);
+	$$end{$n} = $ret;
+	print FLOG "$n      $$start{$n}      $$end{$n}     $$type{$n}\n";
 }
 close(PARTED);
 }
 
+sub decode_Bsuf {
+
+my $size  = shift;
+my $unit  = shift;
+my $ret = 0;
+
+#print FLOG "decode_Bsuf input: $size / $unit ";
+if ($size =~ /K[B]*$/i) {
+	$size =~ s/K[B]*$//i;
+	$size *= 1024;
+} elsif ($size =~ /M[B]*$/i) {
+	$size =~ s/M[B]*$//i;
+	$size *= 1048576;
+} elsif ($size =~ /G[B]*$/i) {
+	$size =~ s/G[B]*$//i;
+	$size *= 1073741824;
+} elsif ($size =~ /T[B]*$/i) {
+	$size =~ s/T[B]*$//i;
+	$size *= 1099511627776;
+} else {
+	# Nothing to do
+}
+$ret = $size / $unit;
+#print FLOG " - output : $size => $ret\n";
+return($ret);
+}
+
+
 # Based on Version 2.4  27-Sep-1996  Charles Bailey  bailey@genetics.upenn.edu
 # in Basename.pm
 
@@ -546,7 +605,7 @@ sub which_type {
 my $device = shift;
 my $type = "";
 
-open (FDISK, "$fdisk -l $device |") || die "Unable to read from $fdisk";
+open (FDISK, "$fdisk -l $device 2>/dev/null |") || die "Unable to read from $fdisk";
 while (<FDISK>) {
 	if ($_ =~ /EFI GPT/) {
 		$type= "gpt";
